﻿using System;
using System.Reflection;
using System.Reflection.Emit;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MetadataLib;

namespace MetadataLib.Test
{
    [TestClass]
    public class MetadataUtilTest
    {
        [TestMethod]
        public  void TestLocalStaticProperty()
        {
            var mi = MetadataUtil.GetSetterName(() => { Class1.MyStaticProp = OpCodes.Call.Value; });
            Assert.AreEqual("set_MyStaticProp", mi.Name);
        }

        [TestMethod]
        public void TestLocalInstanceProperty()
        {
            Class1 c1 = null;
            var mi = MetadataUtil.GetSetterName(() => { c1.MyInstanceProp = OpCodes.Callvirt.Value; });
            Assert.AreEqual("set_MyInstanceProp", mi.Name);
        }

        [TestMethod]
        public void TestLocalNestedStaticProperty()
        {
            var mi = MetadataUtil.GetSetterName(() => { Class1.Class2.MyNestedStaticProp = OpCodes.Call.Value; });
            Assert.AreEqual("set_MyNestedStaticProp", mi.Name);
        }

        [TestMethod]
        public void TestLocalNestedInstanceProperty()
        {
            Class1.Class2 c2 = null;
            var mi = MetadataUtil.GetSetterName(() => { c2.MyNestedInstanceProp = OpCodes.Callvirt.Value; });
            Assert.AreEqual("set_MyNestedInstanceProp", mi.Name);
        }

        [TestMethod]
        public void TestReferecedStaticProperty()
        {
            var mi = MetadataUtil.GetSetterName(() => { DemoLib.Class1.SomeStaticProp = null; });
            Assert.AreEqual("set_SomeStaticProp", mi.Name);
        }

        [TestMethod]
        public void TestReferecedInstanceProperty()
        {
            DemoLib.Class1 c1 = null;
            var mi = MetadataUtil.GetSetterName(() => { c1.SomeInstanceProp = null; });
            Assert.AreEqual("set_SomeInstanceProp", mi.Name);
        }

        [TestMethod]
        public void TestReferencedNestedStaticProperty()
        {
            var mi = MetadataUtil.GetSetterName(() => { DemoLib.Class1.Class2.AnotherStaticProp = 1; });
            Assert.AreEqual("set_AnotherStaticProp", mi.Name);
        }

        [TestMethod]
        public void TestReferencedNestedInstanceProperty()
        {
            DemoLib.Class1.Class2 c2 = null;
            var mi = MetadataUtil.GetSetterName(() => { c2.AnotherInstanceProp = 1; });
            Assert.AreEqual("set_AnotherInstanceProp", mi.Name);
        }

        [TestMethod]
        public void TestReferencedNestedStaticProperty2()
        {
            var mi = MetadataUtil.GetSetterName(() => { DemoLib.Class1.Class2.Class3.YetAnotherStaticProp = null; });
            Assert.AreEqual("set_YetAnotherStaticProp", mi.Name);
        }

        [TestMethod]
        public void TestReferencedNestedInstanceProperty2()
        {
            DemoLib.Class1.Class2.Class3 c3 = null;
            var mi = MetadataUtil.GetSetterName(() => { c3.YetAnotherInstanceProp = null; });
            Assert.AreEqual("set_YetAnotherInstanceProp", mi.Name);
        }
    }
}
